import click

from elementalcms.core import ElementalContext
from elementalcms.services.pages import GetAll


class List:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, drafts=False):
        result = GetAll(self.context.cms_db_context).execute(drafts)
        if result.is_failure():
            click.echo('There are no pages to list. Create your first one by using the [pages create] command.')
            return

        # TODO: Add repos difference indicators

        for spec in result.value():
            click.echo(f'{spec["name"]} -> {spec["title"]}')



import click
from cloup import constraint, option, command, pass_context, argument
from cloup.constraints import RequireExactly

from .mediacommands import List, Pull, Push, Delete


class Media(click.Group):

    def __init__(self):
        super(Media, self).__init__()
        self.name = 'media'
        self.add_command(self.list)
        self.add_command(self.pull)
        self.add_command(self.push)
        self.add_command(self.delete)

    @staticmethod
    @command(name='list',
             help='List media files.')
    @option('--all',
            is_flag=True,
            help='List all media files.')
    @option('--folder',
            '-f',
            nargs=1,
            multiple=True,
            help='Name of the folder to be listed. For example: list -f default/')
    @constraint(RequireExactly(1), ['all', 'folder'])
    @pass_context
    def list(ctx, **params):
        if params['all']:
            List(ctx).exec(None)
            return
        List(ctx).exec(params['folder'][0])

    @staticmethod
    @command(name='pull',
             help='Pull media file(s) from GCS.')
    @option('--all',
            is_flag=True,
            help='Pull all media files into the local media folder.')
    @option('--folder',
            '-f',
            nargs=1,
            multiple=True,
            help='Name of the folder to be pulled. For example: pull -f default/')
    @constraint(RequireExactly(1), ['all', 'folder'])
    @pass_context
    def pull(ctx, **params):
        if params['all']:
            return Pull(ctx).exec('*')
        return Pull(ctx).exec(params['folder'])

    @staticmethod
    @command(name='push')
    @argument('pattern')
    @pass_context
    def push(ctx, pattern):
        """Push media file(s) to GCS.

        You must use a search PATTERN to identify the files to be pushed.

        Samples (assuming media is our local media folder):

        \b
        media/*.*
        media/*.txt
        media/*.svg
        media/**/images
        """
        Push(ctx).exec(pattern)

    @staticmethod
    @command(name='delete')
    @argument('file')
    @pass_context
    def delete(ctx, file):
        """Delete especified file from GCS."""
        Delete(ctx).exec(file)



class UseCaseResult(object):
    def is_failure(self):
        raise Exception("is_failure method needs to be implemented")

    def value(self):
        raise Exception("is_failure method needs to be implemented")


class Failure(UseCaseResult):
    def __init__(self, v): self.v = v
    def is_failure(self): return True
    def value(self): return self.v


class AccessDenied(Failure):
    def __init__(self):
        super().__init__(None)


class NoResult(Failure):
    def __init__(self):
        super().__init__(None)


class Success(UseCaseResult):
    def __init__(self, v): self.v = v
    def is_failure(self): return False
    def value(self): return self.v



import functools
import inspect
from flask import Flask


class ActionsMapper:

    __app: Flask

    def __init__(self, api: Flask):
        self.__app = api

    def register_actions(self, controllers) -> []:
        registered_views = []
        for controller in controllers:
            predicate = inspect.ismethod
            all_members = inspect.getmembers(controller, predicate=predicate)
            for fn_name, value in all_members:
                if 'is_action' in dir(value):
                    controller_name = type(controller).__name__
                    view = getattr(controller, fn_name)

                    def make_func(fn):
                        @functools.wraps(fn)
                        def inner(*args, **kwargs):
                            return fn(*args, **kwargs)

                        return inner

                    view = make_func(view)

                    endpoint = self.__build_endpoint(view, controller_name, fn_name)
                    self.__app.add_url_rule(view.route,
                                            endpoint,
                                            view,
                                            methods=[view.method])
                    registered_views.append(view)
        return registered_views

    @staticmethod
    def __build_endpoint(view, controller_name, fn_name):
        if view.endpoint is not None:
            return view.endpoint
        return '({}) [{}.{}]'.format(view.method, controller_name, fn_name)



from elementalcms.core import MongoDbContext
from elementalcms.persistence.repositories import GenericRepository


class SessionsRepository(GenericRepository):

    coll_name = 'sessions'

    def __init__(self, db_context: MongoDbContext):
        super().__init__(db_context, 'sessions')

    def create_expiration_index(self):
        index_info = self._coll.index_information()
        if 'ix.expiration' in index_info:
            return
        self._coll.create_index('expiration',
                                name='ix.expiration',
                                expireAfterSeconds=0)



import secrets
from functools import wraps
from http import HTTPStatus

from flask import Blueprint, session, request, Response

identity = Blueprint('identity', __name__)


@identity.app_context_processor
def gateway_token_processor():
    if 'gatewayToken' not in session:
        session['gatewayToken'] = secrets.token_hex(nbytes=16)
    return dict(gateway_token=session.get('gatewayToken'))


def gateway_token_required(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        session_gateway_token = session.get('gatewayToken', None)
        if session_gateway_token is None:
            return Response(status=HTTPStatus.UNAUTHORIZED)
        gateway_token = request.headers.get('X-Gateway-Token', None)
        if gateway_token is None:
            return Response(status=HTTPStatus.UNAUTHORIZED)
        if gateway_token != session_gateway_token:
            return Response(status=HTTPStatus.UNAUTHORIZED)
        return f(*args, **kwargs)
    return wrapper


@identity.route('/identity/', methods=['POST'])
@gateway_token_required
def set_user_identity():
    if not request.data:
        return Response(status=HTTPStatus.BAD_REQUEST)
    session['userIdentity'] = request.json
    return {}, HTTPStatus.CREATED


@identity.route('/identity/', methods=['DELETE'])
@gateway_token_required
def remove_user_identity():
    if 'userIdentity' in session:
        session.pop('userIdentity')
    return {}, HTTPStatus.OK



from elementalcms.extends import Controller


class Applet:

    __controllers: [Controller] = []

    def __init__(self, controllers: [Controller]):
        self.__controllers = controllers

    def get_controllers(self) -> [Controller]:
        return self.__controllers



from typing import Optional

from cloup import constraint, option, command, pass_context, Group
from cloup.constraints import RequireExactly

from .globaldepscommands import List, Create, Push, Pull, Remove


class GlobalDeps(Group):

    def __init__(self):
        super(GlobalDeps, self).__init__()
        self.name = 'global-deps'
        self.add_command(self.list)
        self.add_command(self.create)
        self.add_command(self.push)
        self.add_command(self.pull)
        self.add_command(self.remove)

        # TODO: Add command to find differences between local workspace and CMS database

    @staticmethod
    @command(name='list',
             help='Display global dependencies list.')
    @pass_context
    def list(ctx):
        List(ctx).exec()

    @staticmethod
    @command(name='create',
             help='Create a new global dependency at local workspace.')
    @option('--dep',
            '-d',
            required=True,
            nargs=2,
            help='Name and type for the global dep(s) to be created; name must be unique, lowercase and it can not '
                 'contains special characters, and the type can be any of: text/css application/javascript or module. '
                 'For example create --dep jquery application/javascript')
    @pass_context
    def create(ctx, dep):
        Create(ctx).exec(dep[0], dep[1])

    @staticmethod
    @command(name='push',
             help='Push global dep(s) specs to the CMS database.')
    @option('--all',
            is_flag=True,
            help='Push all global dependencies.')
    @option('--dep',
            '-d',
            nargs=2,
            multiple=True,
            help='Name and type for the global dep(s) to be pushed. '
                 'For example: push -d bootstrap text/css -d bootstrap application/javascript')
    @constraint(RequireExactly(1), ['all', 'dep'])
    @pass_context
    def push(ctx, **params) -> [str]:
        if params['all']:
            return Push(ctx).exec('*')
        return Push(ctx).exec(params['dep'])

    @staticmethod
    @command(name='pull',
             help='Pull global dep(s) specs from the CMS database.')
    @option('--all',
            is_flag=True,
            help='Pull all global dependencies.')
    @option('--dep',
            '-d',
            nargs=2,
            multiple=True,
            help='Name and type for the global dep(s) to be pulled. '
                 'For example: pull -d bootstrap application/javascript')
    @constraint(RequireExactly(1), ['all', 'dep'])
    @pass_context
    def pull(ctx, **params) -> [str]:
        if params['all']:
            return Pull(ctx).exec('*')
        return Pull(ctx).exec(params['dep'])

    @staticmethod
    @command(name='remove', help='Remove global dependency from the CMS database.')
    @option('--dep',
            '-d',
            required=True,
            nargs=2,
            help='Name and type for the global dependency to be removed. '
                 'For example: remove --dep bootstrap application/javascript')
    @pass_context
    def remove(ctx, dep) -> Optional[str]:
        return Remove(ctx).exec(dep[0], dep[1])



import click
from google.cloud import storage
from google.cloud.storage import Bucket

from elementalcms.core import ElementalContext


class Delete:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, file_name):

        if self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO:
            client = storage.Client.from_service_account_info(self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO)
        else:
            client = storage.Client()

        bucket: Bucket = client.bucket(self.context.cms_core_context.STATIC_BUCKET)
        blob = bucket.blob(file_name)
        blob.delete()
        click.echo(f'{file_name} removed successfuly.')



from elementalcms.persistence.repositories import SessionsRepository
from elementalcms.services import UseCaseResult, Success
from elementalcms.core import MongoDbContext


class CreateExpirationIndex:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self) -> UseCaseResult:
        repo = SessionsRepository(self.__db_context)
        repo.create_expiration_index()
        return Success(True)



from typing import Tuple, Optional

import click
from cloup import constraint, option, command, pass_context
from cloup.constraints import RequireExactly

from .pagescommands import Create, Remove, Push, Pull, List, Publish, Unpublish


class Pages(click.Group):

    def __init__(self):
        super(Pages, self).__init__()
        self.name = 'pages'
        self.add_command(self.list)
        self.add_command(self.create)
        self.add_command(self.remove)
        self.add_command(self.push)
        self.add_command(self.pull)
        self.add_command(self.publish)
        self.add_command(self.unpublish)

        # TODO: Add command to find differences between local workspace and CMS database

    @staticmethod
    @command(name='list',
             help='Display pages list.')
    @option('--drafts',
            is_flag=True,
            help='Display the draft pages list (false by default).')
    @pass_context
    def list(ctx, drafts):
        List(ctx).exec(drafts)

    @staticmethod
    @command(name='create', help='Create a new page on your local workspace.')
    @option('--page',
            '-p',
            nargs=2,
            required=True,
            help='Name and language for the page to be created. Name must be unique, lowercased and it can not '
                 'contains special characters but - or _. For example: create -p home en')
    @pass_context
    def create(ctx, page):
        Create(ctx).exec(page)

    @staticmethod
    @command(name='push',
             help='Push page(s) spec(s) and content(s) to the CMS database. '
                  'All pushed pages are stored initially at the draft collection.')
    @option('--all',
            is_flag=True,
            help='Push all pages.')
    @option('--page',
            '-p',
            nargs=2,
            multiple=True,
            help='Name and language for the page to be pushed. For example: push -p home en -p home es')
    @constraint(RequireExactly(1), ['all', 'page'])
    @pass_context
    def push(ctx, **params) -> [Tuple]:
        if params['all']:
            return Push(ctx).exec('*')
        return Push(ctx).exec(params['page'])

    @staticmethod
    @command(name='publish',
             help='Publish one especific localized page.')
    @option('--page',
            '-p',
            nargs=2,
            required=True,
            help='Page name and language. For example: publish -p home es')
    @pass_context
    def publish(ctx, page) -> [Tuple]:
        return Publish(ctx).exec(page)

    @staticmethod
    @command(name='unpublish',
             help='Unpublish one especific page localized page.')
    @option('--page',
            '-p',
            nargs=2,
            required=True,
            help='Page name and language. For example: unpublish -p sign-in en')
    @pass_context
    def unpublish(ctx, page) -> [Tuple]:
        return Unpublish(ctx).exec(page)

    @staticmethod
    @command(name='pull',
             help='Pull page(s) spec(s) and content(s) from the CMS database.')
    @option('--all',
            is_flag=True,
            help='Pull all pages.')
    @option('--page',
            '-p',
            nargs=2,
            multiple=True,
            help='Name and language for the page to be pulled. For example: pull -p home en -p home es')
    @constraint(RequireExactly(1), ['all', 'page'])
    @option('--drafts',
            is_flag=True,
            help='Use this option to pull the page(s) draft version.')
    @pass_context
    def pull(ctx, **params) -> [Tuple]:
        if params['all']:
            return Pull(ctx).exec('*', params['drafts'])
        return Pull(ctx).exec(params['page'], params['drafts'])

    @staticmethod
    @command(name='remove', help='Remove unpublished pages. This command removes the page draft version; if you '
                                 'want to remove the page published version you must use the pages unpublish command.')
    @option('--page',
            '-p',
            nargs=2,
            required=True,
            help='Name and language for the page to be pulled. For example: remove -p home es')
    @pass_context
    def remove(ctx, page) -> Optional[Tuple]:
        return Remove(ctx).exec(page)



def get(route: str, **kwargs):
    def wrapper(function):
        function.is_action = True
        function.route = route
        function.method = 'GET'
        function.endpoint = None

        if 'endpoint' in kwargs.keys():
            function.endpoint = kwargs['endpoint']
        return function

    return wrapper


def post(route: str, **kwargs):
    def wrapper(function):
        function.is_action = True
        function.route = route
        function.method = 'POST'
        function.endpoint = None

        if 'endpoint' in kwargs.keys():
            function.endpoint = kwargs['endpoint']

        return function

    return wrapper


def put(route: str, **kwargs):
    def wrapper(function):
        function.is_action = True
        function.route = route
        function.method = 'PUT'
        function.endpoint = None

        if 'endpoint' in kwargs.keys():
            function.endpoint = kwargs['endpoint']

        return function

    return wrapper


def patch(route: str, **kwargs):
    def wrapper(function):
        function.is_action = True
        function.route = route
        function.method = 'PATCH'
        function.endpoint = None

        if 'endpoint' in kwargs.keys():
            function.endpoint = kwargs['endpoint']

        return function

    return wrapper


def delete(route: str, **kwargs):
    def wrapper(function):
        function.is_action = True
        function.route = route
        function.method = 'DELETE'
        function.endpoint = None

        if 'endpoint' in kwargs.keys():
            function.endpoint = kwargs['endpoint']

        return function

    return wrapper



import time
import os
from shutil import copyfile
from typing import Optional, Tuple
import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services.snippets import GetMe, GetAll


class Pull:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, snippets) -> [Tuple]:
        if isinstance(snippets, str):
            get_all_result = GetAll(self.context.cms_db_context).execute()
            snippets_tuples = [] if get_all_result.is_failure() else [(item['name'])
                                                                      for item in get_all_result.value()]
            if len(snippets_tuples) == 0:
                click.echo('There are no snippets to pull.')
                return []
        else:
            snippets_tuples = snippets
            # TODO: Support file names with paths

        backups_filepaths = []
        for name in snippets_tuples:
            get_me_result = GetMe(self.context.cms_db_context).execute(name)
            snippet = get_me_result.value()
            if snippet is None:
                click.echo(f'Snippet {name} does not exist.')
                continue
            folder_path = self.context.cms_core_context.SNIPPETS_FOLDER
            if not os.path.exists(folder_path):
                os.makedirs(folder_path)
            spec_filepath = f'{folder_path}/{name}.json'
            content_filepath = f'{folder_path}/{name}.html'
            backup_filespaths = self.build_local_backups(folder_path, spec_filepath, content_filepath, name)
            html_content = snippet.pop('content', '')
            spec_file = open(spec_filepath, mode='w', encoding='utf-8')
            spec_file.write(json_util.dumps(snippet, indent=4))
            spec_file.close()
            content_file = open(content_filepath, mode='w', encoding='utf-8')
            content_file.write(html_content)
            content_file.close()
            click.echo(f'Snippet {name} pulled successfully.')
            if backup_filespaths is not None:
                backups_filepaths.append(backup_filespaths)
        return backups_filepaths

    @staticmethod
    def build_local_backups(folder_path, spec_filepath, content_filepath, clean_file_name) -> Optional[Tuple]:
        click.echo('Building local backups...')
        suffix = round(time.time())
        backups_folder_path = f'{folder_path}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        if os.path.exists(spec_filepath) and os.path.exists(content_filepath):
            spec_backup_filepath = f'{backups_folder_path}/{clean_file_name}-{suffix}.json'
            copyfile(spec_filepath, spec_backup_filepath)
            content_backup_filepath = f'{backups_folder_path}/{clean_file_name}-{suffix}.html'
            copyfile(content_filepath, content_backup_filepath)
            return spec_backup_filepath, content_backup_filepath
        return None



import time
import os
from shutil import copyfile
from typing import Optional
import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services.global_deps import GetMe, GetAll


class Pull:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, deps) -> [str]:
        if isinstance(deps, str):
            get_all_result = GetAll(self.context.cms_db_context).execute()
            deps_tuples = [] if get_all_result.is_failure() else [(item['name'], item['type'])
                                                                  for item in get_all_result.value()]
            if len(deps_tuples) == 0:
                click.echo('There are no global dependencies to pull.')
                return []
        else:
            deps_tuples = deps
            # TODO: Support file names with paths

        root_folder_path = self.context.cms_core_context.GLOBAL_DEPS_FOLDER

        backups_filepaths = []
        for dep_tuple in deps_tuples:
            name = dep_tuple[0]
            _type = dep_tuple[1]
            get_me_result = GetMe(self.context.cms_db_context).execute(name, _type)
            dep = get_me_result.value()
            if dep is None:
                click.echo(f'Global dependency {name} ({_type}) does not exist.')
                continue
            type_folder_name = _type.replace('/', '_')
            folder_path = f'{root_folder_path}/{type_folder_name}'
            if not os.path.exists(folder_path):
                os.makedirs(folder_path)
            spec_filepath = f'{folder_path}/{name}.json'
            spec_backup_filepath = self.build_local_backup(folder_path, spec_filepath, name)
            spec_file = open(spec_filepath, mode='w', encoding='utf-8')
            spec_file.write(json_util.dumps(dep, indent=4))
            spec_file.close()
            click.echo(f'Global dependency {name} ({_type}) pulled successfully.')
            if spec_backup_filepath is not None:
                backups_filepaths.append(spec_backup_filepath)
        return backups_filepaths

    @staticmethod
    def build_local_backup(folder_path, spec_filepath, clean_file_name) -> Optional[str]:
        click.echo('Building local backup...')
        suffix = round(time.time())
        backups_folder_path = f'{folder_path}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        if os.path.exists(spec_filepath):
            spec_backup_filepath = f'{backups_folder_path}/{clean_file_name}-local-{suffix}.json'
            copyfile(spec_filepath, spec_backup_filepath)
            return spec_backup_filepath
        return None



from elementalcms.persistence.repositories import SnippetsRepository
from elementalcms.services import UseCaseResult, NoResult, Success, Failure
from elementalcms.core import MongoDbContext


class GetMe:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, name) -> UseCaseResult:
        repo = SnippetsRepository(self.__db_context)
        result = repo.find({'name': name}, page=0, page_size=10)
        if result['total'] == 0:
            return NoResult()
        if result['total'] > 1:
            return Failure({'duplicatedSnippet': True})
        return Success(result['items'][0])



from elementalcms.persistence.repositories import SnippetsRepository
from elementalcms.services import UseCaseResult, NoResult, Success
from elementalcms.core import MongoDbContext


class GetAll:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self) -> UseCaseResult:
        repo = SnippetsRepository(self.__db_context)
        snippets = []
        page = 0
        page_size = 50
        while True:
            result = repo.find(page=page, page_size=page_size)
            total = result['total']
            if total == 0:
                return NoResult()
            snippets.extend(result['items'])
            if len(snippets) >= total:
                break
            page += 1
        return Success(snippets)



import datetime
from elementalcms.persistence.repositories import SnippetsRepository
from elementalcms.services import UseCaseResult, Success
from elementalcms.core import MongoDbContext


class UpdateOne:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, _id, snippet) -> UseCaseResult:
        repo = SnippetsRepository(self.__db_context)
        if '_id' in snippet:
            del snippet['_id']
        snippet['lastModifiedAt'] = datetime.datetime.utcnow()
        success = repo.replace_one(_id, snippet, True)
        return Success(success)



import datetime
import os

import click
from bson import ObjectId, json_util

from elementalcms import ElementalContext


class Create:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, page_tuple):

        # TODO: Validate naming constraints

        name = page_tuple[0]
        lang = page_tuple[1]
        if lang not in self.context.cms_core_context.LANGUAGES:
            click.echo(f'"{lang}" language is not supported')
            return
        root_folder_path = self.context.cms_core_context.PAGES_FOLDER
        folder_path = f'{root_folder_path}/{lang}'
        if not os.path.exists(folder_path):
            os.makedirs(folder_path)
        spec_filepath = f'{folder_path}/{name}.json'
        content_filepath = f'{folder_path}/{name}.html'
        if os.path.exists(spec_filepath):
            click.echo(f'{name} ({lang}) already exist.')
            return
        spec = {
            '_id': ObjectId(),
            'name': name,
            'language': lang,
            'title': f'{name} page',
            'description': '',
            'isHome': False,
            'requiresUserIdentity': False,
            'redirectUsersTo': '',
            'cssDeps': [],
            'jsDeps': [],
            'createdAt': datetime.datetime.utcnow(),
            'lastModifiedAt': datetime.datetime.utcnow()
        }
        spec_file = open(spec_filepath, mode='w', encoding='utf-8')
        spec_file.write(json_util.dumps(spec, indent=4))
        spec_file.close()
        content_file = open(content_filepath, mode='w', encoding='utf-8')
        content_file.write('<div></div>')
        content_file.close()
        click.echo(f'{folder_path}/{name}.json|html files has been created successfully.')



import click

from elementalcms.core import ElementalContext
from elementalcms.services.snippets import GetAll


class List:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self):
        result = GetAll(self.context.cms_db_context).execute()
        if result.is_failure():
            click.echo('There are no snippets to list. Create your first one by using the [snippets create] command.')
            return

        # TODO: Add repos difference indicators

        for snippet in result.value():
            click.echo(f'{snippet["name"]}')



import os
import time
from typing import Optional, Tuple

import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services.pages import GetMe, RemoveOne


class Remove:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, page_tuple) -> Optional[Tuple]:

        name = page_tuple[0]
        lang = page_tuple[1]

        get_page_result = GetMe(self.context.cms_db_context).execute(name, lang, False, False)
        page = get_page_result.value()

        if page is not None:
            click.echo(f'{name} ({lang}) has to be unpublished first, in order to be removed.')
            return None

        get_draft_result = GetMe(self.context.cms_db_context).execute(name, lang, True, False)
        draft = get_draft_result.value()

        if draft is None:
            click.echo(f'{name} ({lang}) does not have a draft version.')
            return None

        RemoveOne(self.context.cms_db_context).execute(draft['_id'], True)
        click.echo(f'{name} ({lang}) removed successfully.')
        return self.build_draft_backups(draft)

    def build_draft_backups(self, draft) -> Tuple:
        click.echo('Building backup...')
        html_content = draft.pop('content', '')
        backups_folder_path = f'{self.context.cms_core_context.PAGES_FOLDER}/{draft["language"]}/.bak'
        os.makedirs(backups_folder_path, exist_ok=True)
        sufix = round(time.time())
        spec_backup_filepath = f'{backups_folder_path}/{draft["name"]}-draft-{sufix}.json'
        spec_backup_file = open(spec_backup_filepath, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(draft, indent=4))
        spec_backup_file.close()
        content_backup_filepath = f'{backups_folder_path}/{draft["name"]}-draft-{sufix}.html'
        content_backup_file = open(content_backup_filepath, mode='w', encoding='utf-8')
        content_backup_file.write(html_content)
        content_backup_file.close()
        return spec_backup_filepath, content_backup_filepath



from elementalcms.persistence.repositories import SessionsRepository
from elementalcms.services import UseCaseResult, Success, Failure
from elementalcms.core import MongoDbContext


class UpsertMe:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, value) -> UseCaseResult:
        repo = SessionsRepository(self.__db_context)
        find_result = repo.find({'sid': value['sid']})
        if find_result['total'] == 0:
            new_id = repo.insert_one(value)
            return Success({'id': new_id})
        if find_result['total'] > 1:
            return Failure({'duplicatedSession': True})
        current = find_result['items'][0]
        replace_one_result = repo.replace_one(current['_id'], value, True)
        return Success(replace_one_result)



from elementalcms.persistence.repositories import PagesRepository, DraftsRepository
from elementalcms.services import UseCaseResult, NoResult, Success
from elementalcms.core import MongoDbContext


class GetOne:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, _id, draft=False) -> UseCaseResult:
        if draft:
            repo = DraftsRepository(self.__db_context)
        else:
            repo = PagesRepository(self.__db_context)
        page = repo.find_one(_id)
        if page is None:
            return NoResult()
        return Success(page)



import time
import os
from typing import Tuple, Optional
import click
from bson import json_util, ObjectId

from elementalcms.core import ElementalContext
from elementalcms.services.snippets import GetOne, UpdateOne


class Push:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, snippets) -> [Tuple]:

        folder_path = self.context.cms_core_context.SNIPPETS_FOLDER

        if isinstance(snippets, str):
            snippets_tuples = []
            for r, d, f in os.walk(folder_path):
                if '.bak' in r:
                    continue
                for file in f:
                    if '.json' not in file:
                        continue
                    snippets_tuples.append((file.split('.')[0]))
            if len(snippets_tuples) == 0:
                click.echo('There are no snippets to push.')
                return
        else:
            snippets_tuples = snippets
            # TODO: Support file names with paths

        backups_filepaths = []
        for name in snippets_tuples:
            spec_filepath = f'{folder_path}/{name}.json'
            content_filepath = f'{folder_path}/{name}.html'
            if not os.path.exists(spec_filepath):
                click.echo(f'There is no spec file for {name} snippet.')
                return
            if not os.path.exists(content_filepath):
                click.echo(f'There is no content file for {name} snippet.')
                return
            with open(spec_filepath) as spec_file:
                try:
                    snippet = json_util.loads(spec_file.read())
                except Exception as e:
                    click.echo(f'Invalid spec for {name} - {e}')
                    continue
                if '_id' not in snippet:
                    click.echo(f'Missing spec _id for: {name}.')
                    return
                if not ObjectId.is_valid(snippet['_id']):
                    click.echo(f'Invalid spec _id for: {name}.')
                    return
                if 'name' not in snippet:
                    click.echo(f'Missing spec name for: {name}.')
                    return
                if snippet['name'] != name:
                    click.echo(f'Invalid spec name for: {name}.')
                    return
                with open(content_filepath) as content_file:
                    snippet['content'] = content_file.read()
                    _id = snippet['_id']
                    backup_filepaths = self.build_backups(_id)
                    UpdateOne(self.context.cms_db_context).execute(_id, snippet)
                    click.echo(f'Snippet {name} pushed successfully.')
                    if backup_filepaths is not None:
                        backups_filepaths.append(backup_filepaths)
        return backups_filepaths

    def build_backups(self, _id) -> Optional[Tuple]:
        get_one_result = GetOne(self.context.cms_db_context).execute(_id)
        if get_one_result.is_failure():
            return None
        click.echo('Building backups...')
        backups_folder_path = f'{self.context.cms_core_context.SNIPPETS_FOLDER}/.bak'
        os.makedirs(backups_folder_path, exist_ok=True)
        snippet = get_one_result.value()
        html_content = snippet.pop('content', '')
        sufix = round(time.time())
        spec_backup_filepath = f'{backups_folder_path}/{snippet["name"]}-{sufix}.json'
        spec_backup_file = open(spec_backup_filepath, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(snippet, indent=4))
        spec_backup_file.close()
        content_backup_filepath = f'{backups_folder_path}/{snippet["name"]}-{sufix}.html'
        content_backup_file = open(content_backup_filepath, mode='w', encoding='utf-8')
        content_backup_file.write(html_content)
        content_backup_file.close()
        return spec_backup_filepath, content_backup_filepath



import time
import os
from typing import Optional, Tuple

from deepdiff import DeepDiff
import click
from bson import json_util
from elementalcms.core import ElementalContext
from elementalcms.services.pages import GetMe, UpdateOne


class Publish:
    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, page_tuple) -> Optional[Tuple]:

        name = page_tuple[0]
        lang = page_tuple[1]

        get_draft_result = GetMe(self.context.cms_db_context).execute(name, lang, True, False)
        draft = get_draft_result.value()

        if draft is None:
            click.echo(f'{name} ({lang}) does not have a draft version.')
            return

        get_page_result = GetMe(self.context.cms_db_context).execute(name, lang, False, False)
        page = get_page_result.value()

        backup_filepaths = None
        if page is not None:
            if DeepDiff(draft, page) == {}:
                click.echo(f'{name} ({lang}) is already published.')
                return
            backup_filepaths = self.build_page_backup(page)
            _id = page['_id']
        else:
            _id = draft['_id']

        UpdateOne(self.context.cms_db_context).execute(_id, draft)
        click.echo(f'{name} ({lang}) published successfully.')
        return backup_filepaths

    def build_page_backup(self, page) -> Optional[Tuple]:
        click.echo('Building backups...')
        backups_folder_path = f'{self.context.cms_core_context.PAGES_FOLDER}/{page["language"]}/.bak'
        os.makedirs(backups_folder_path, exist_ok=True)
        html_content = page.pop('content', '')
        sufix = round(time.time())
        spec_backup_filepath = f'{backups_folder_path}/{page["name"]}-{sufix}.json'
        spec_backup_file = open(spec_backup_filepath, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(page, indent=4))
        spec_backup_file.close()
        content_backup_filepath = f'{backups_folder_path}/{page["name"]}-{sufix}.html'
        content_backup_file = open(content_backup_filepath, mode='w', encoding='utf-8')
        content_backup_file.write(html_content)
        content_backup_file.close()
        return spec_backup_filepath, content_backup_filepath



import re
import pycountry
from flask import render_template, Blueprint, request, g, current_app, redirect, abort, session, \
    render_template_string, url_for

from elementalcms.services import UseCaseResult
from elementalcms.services.pages import GetHome, GetMe
from elementalcms.services.snippets import GetMany


presenter = Blueprint('presenter', __name__, template_folder='templates')


@presenter.before_request
def before_request():
    lang_code = g.get('lang_code', None)
    if lang_code is None and current_app.config['LANGUAGE_MODE'] == 'single':
        return
    if lang_code is None:
        abort(404)
    if lang_code in current_app.config['LANGUAGES']:
        return
    return redirect(request.full_path.replace(lang_code,
                                              session.get('langCode', current_app.config["DEFAULT_LANGUAGE"])))


@presenter.url_defaults
def add_language_code(endpoint, values):
    values.setdefault('lang_code', session.get('langCode', current_app.config["DEFAULT_LANGUAGE"]))


@presenter.url_value_preprocessor
def pull_lang_code(endpoint, values):

    lang_code = values.get('lang_code', None)
    if lang_code is None:
        return

    country = pycountry.languages.get(alpha_2=lang_code)
    if country is None:
        return

    g.lang_code = lang_code


@presenter.route('/', methods=['GET'])
def index(lang_code: str = None):

    draft = request.args.get('draft')
    lang_mode = current_app.config['LANGUAGE_MODE']

    if lang_code is None and lang_mode == 'single':
        lang_code = session.get('langCode', current_app.config['DEFAULT_LANGUAGE'])

    result: UseCaseResult = GetHome(current_app.config['CMS_DB_CONTEXT']).execute(lang_code,
                                                                                  draft=(draft == '1'))
    if result.is_failure():
        abort(404)

    requires_user_identity = result.value().get('requiresUserIdentity', False)
    redirect_users_to = result.value().get('redirectUsersTo', '').strip() or None
    has_user_identity = session.get('userIdentity', None)

    if redirect_users_to and has_user_identity:
        if draft == '1':
            if redirect_users_to == 'index':
                return redirect(url_for('presenter.index',
                                        lang_code=None if lang_mode == 'single' else lang_code,
                                        draft=1),
                                301)
            return redirect(url_for('presenter.render',
                                    lang_code=None if lang_mode == 'single' else lang_code,
                                    slug=redirect_users_to,
                                    draft=1))
        if redirect_users_to == 'index':
            return redirect(url_for('presenter.index',
                                    lang_code=None if lang_mode == 'single' else lang_code),
                            301)
        return redirect(url_for('presenter.render',
                                lang_code=None if lang_mode == 'single' else lang_code,
                                slug=redirect_users_to))

    if requires_user_identity and not has_user_identity:
        abort(401)

    session['langCode'] = lang_code
    return render_template('presenter/index.html',
                           page=get_page_model(result.value()))


@presenter.route('/<slug>/', methods=['GET'])
def render(slug: str, lang_code: str = None):

    draft = request.args.get('draft')
    lang_mode = current_app.config['LANGUAGE_MODE']

    if lang_code is None and lang_mode == 'single':
        lang_code = session.get('langCode', current_app.config['DEFAULT_LANGUAGE'])

    result: UseCaseResult = GetMe(current_app.config['CMS_DB_CONTEXT']).execute(slug,
                                                                                lang_code,
                                                                                draft=(draft == '1'))
    if result.is_failure():
        abort(404)

    requires_user_identity = result.value().get('requiresUserIdentity', False)
    redirect_users_to = result.value().get('redirectUsersTo', '').strip() or None
    has_user_identity = session.get('userIdentity', None)

    if redirect_users_to and has_user_identity:
        if draft == '1':
            if redirect_users_to == 'index':
                return redirect(url_for('presenter.index',
                                        lang_code=None if lang_mode == 'single' else lang_code,
                                        draft=1))
            return redirect(url_for('presenter.render',
                                    lang_code=None if lang_mode == 'single' else lang_code,
                                    slug=redirect_users_to,
                                    draft=1))
        if redirect_users_to == 'index':
            return redirect(url_for('presenter.index',
                                    lang_code=None if lang_mode == 'single' else lang_code))
        return redirect(url_for('presenter.render',
                                lang_code=None if lang_mode == 'single' else lang_code,
                                slug=redirect_users_to))

    if requires_user_identity and not has_user_identity:
        abort(401)

    session['langCode'] = lang_code
    return render_template('presenter/index.html',
                           page=get_page_model(result.value()))


def get_page_model(page_spec):

    styles = get_styles(page_spec['cssDeps'])

    scripts = get_scripts(page_spec['jsDeps'])
    content = render_template_string(page_spec['content'], page=page_spec)

    snippets_names = re.findall("<!--(.*)-->", content)
    get_many_snippets_result = GetMany(current_app.config['CMS_DB_CONTEXT']).execute(snippets_names)
    snippets = [] if get_many_snippets_result.is_failure() else get_many_snippets_result.value()

    snippets_styles = []
    snippets_scripts = []
    for snippet in snippets:
        snippets_styles += snippet['cssDeps']
        snippets_scripts += snippet['jsDeps']

    styles += get_styles(snippets_styles)
    scripts += get_scripts(snippets_scripts)

    return {
        'title': page_spec['title'],
        'name': page_spec['name'],
        'description': page_spec['description'],
        'content': content,
        'styles': ''.join(styles),
        'scripts': ''.join(scripts)
    }


def get_styles(deps):
    styles = []
    for dep in deps:
        if 'http' in dep['url']:
            styles.append(f'<link rel="stylesheet" type="text/css" href="{dep["url"]}">')
            continue
        href = f'{current_app.config["STATIC_URL"]}/{current_app.config["APP_NAME"]}/{dep["url"]}'
        styles.append(f'<link rel="stylesheet" type="text/css" href="{href}">')
    return styles


def get_scripts(deps):
    scripts = []
    for dep in deps:
        if dep['url'].startswith('http'):
            scripts.append(f'<script src="{dep["url"]}" type="{dep["type"]}"></script>')
            continue
        src = f'{current_app.config["STATIC_URL"]}/{current_app.config["APP_NAME"]}/{dep["url"]}'
        scripts.append(f'<script src="{src}" type="{dep["type"]}"></script>')
    return scripts



import datetime
import os
import click
from bson import ObjectId, json_util

from elementalcms import ElementalContext


class Create:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name, _type):

        # TODO: Validate naming constraints

        if _type not in ['application/javascript',
                         'text/css',
                         'module']:
            click.echo(f'{_type} type is not supported.')
            return
        root_folder_path = self.context.cms_core_context.GLOBAL_DEPS_FOLDER
        type_folder_name = _type.replace('/', '_')
        folder_path = f'{root_folder_path}/{type_folder_name}'
        if not os.path.exists(folder_path):
            os.makedirs(folder_path)
        spec_filepath = f'{folder_path}/{name}.json'
        if os.path.exists(spec_filepath):
            click.echo(f'{name} ({_type}) global dependency already exist.')
            return
        dep = {
            '_id': ObjectId(),
            'order': -1,
            'name': name,
            'type': _type,
            'url': '',
            'meta': {},
            'createdAt': datetime.datetime.utcnow(),
            'lastModifiedAt': datetime.datetime.utcnow()
        }
        spec_file = open(spec_filepath, mode='w', encoding='utf-8')
        spec_file.write(json_util.dumps(dep, indent=4))
        spec_file.close()
        click.echo(f'{spec_filepath} file has been created successfully.')



import time
import os
from shutil import copyfile
from typing import Tuple, Optional

import click
from bson import json_util
from elementalcms.core import ElementalContext
from elementalcms.services.pages import GetMe, GetAll


class Pull:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, pages, drafts: bool) -> [Tuple]:

        if isinstance(pages, str):
            get_all_result = GetAll(self.context.cms_db_context).execute(drafts)
            pages_tuples = [] if get_all_result.is_failure() else [(item['name'], item['language'])
                                                                   for item in get_all_result.value()]
            if len(pages_tuples) == 0:
                click.echo('There are no pages to pull.')
                return []
        else:
            pages_tuples = pages
            # TODO: Support file names with paths

        root_folder_path = self.context.cms_core_context.PAGES_FOLDER

        backups_filepaths = []
        for page_tuple in pages_tuples:
            name = page_tuple[0]
            lang = page_tuple[1]
            get_me_result = GetMe(self.context.cms_db_context).execute(name, lang, drafts, False)
            page = get_me_result.value()
            if page is None:
                if drafts:
                    click.echo(f'{name} ({lang}) does not have a draft version.')
                    continue
                click.echo(f'{name} ({lang}) has not been published.')
                continue
            backup_filepaths = self.build_local_backups(lang, name)
            html_content = page.pop('content')
            folder_path = f'{root_folder_path}/{lang}'
            os.makedirs(folder_path, exist_ok=True)
            spec_filepath = f'{folder_path}/{name}.json'
            spec_file = open(spec_filepath, mode='w', encoding='utf-8')
            spec_file.write(json_util.dumps(page, indent=4))
            spec_file.close()
            content_filepath = f'{folder_path}/{name}.html'
            content_file = open(content_filepath, mode='w', encoding='utf-8')
            content_file.write(html_content)
            content_file.close()
            click.echo(f'{name} ({lang}) pulled successfully.')
            if backup_filepaths is not None:
                backups_filepaths.append(backup_filepaths)
        return backups_filepaths

    def build_local_backups(self, lang, name) -> Optional[Tuple]:
        click.echo('Building local backups...')
        root_folder_path = self.context.cms_core_context.PAGES_FOLDER
        folder_path = f'{root_folder_path}/{lang}'
        backups_folder_path = f'{root_folder_path}/{lang}/.bak'
        os.makedirs(backups_folder_path, exist_ok=True)
        sufix = round(time.time())
        spec_filepath = f'{folder_path}/{name}.json'
        content_filepath = f'{folder_path}/{name}.html'
        spec_backup_filepath = f'{backups_folder_path}/{name}-local-{sufix}.json'
        content_backup_filepath = f'{backups_folder_path}/{name}-local-{sufix}.html'
        if os.path.exists(spec_filepath):
            copyfile(spec_filepath, spec_backup_filepath)
        if os.path.exists(content_filepath):
            copyfile(content_filepath, content_backup_filepath)
        return spec_backup_filepath, content_backup_filepath



import os
import time
from typing import Optional, Tuple
import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services import NoResult
from elementalcms.services.snippets import GetMe, RemoveOne


class Remove:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name) -> Optional[Tuple]:

        get_me_result = GetMe(self.context.cms_db_context).execute(name)
        if get_me_result.is_failure():
            if isinstance(get_me_result, NoResult):
                click.echo(f'Snippet {name} does not exist.')
                return None
            click.echo('Something went wrong and it was not possible to complete the operation.')
            return None

        snippet = get_me_result.value()

        backup_filepaths = self.build_backup(snippet)

        RemoveOne(self.context.cms_db_context).execute(snippet['_id'])
        click.echo(f'Snippet {name} removed successfully.')
        return backup_filepaths

    def build_backup(self, snippet) -> Optional[Tuple]:
        click.echo('Building backup...')
        html_content = snippet.pop('content', '')
        backups_folder_path = f'{self.context.cms_core_context.SNIPPETS_FOLDER}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        sufix = round(time.time())
        spec_backup_filepath = f'{backups_folder_path}/{snippet["name"]}-{sufix}.json'
        spec_backup_file = open(spec_backup_filepath, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(snippet, indent=4))
        spec_backup_file.close()
        content_backup_filepath = f'{backups_folder_path}/{snippet["name"]}-{sufix}.html'
        content_backup_file = open(content_backup_filepath, mode='w', encoding='utf-8')
        content_backup_file.write(html_content)
        content_backup_file.close()
        return spec_backup_filepath, content_backup_filepath



import glob

import click
from google.cloud import storage

from elementalcms.core import ElementalContext


class Push:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, pattern):

        media_folder = self.context.cms_core_context.MEDIA_FOLDER

        if not pattern.startswith(media_folder):
            click.echo(f'We can only push files located at the media folder which right now is {media_folder}')
            return

        if self.context.cms_core_context.MEDIA_BUCKET is None:
            click.echo('MEDIA_BUCKET parameter not found on current settings.')
            return

        files = glob.glob(pattern)
        if len(files) == 0:
            click.echo(f'We found 0 files for {pattern}')
            return

        if self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO:
            client = storage.Client.from_service_account_info(self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO)
        else:
            client = storage.Client()

        bucket = client.bucket(self.context.cms_core_context.MEDIA_BUCKET)
        click.echo(f'Pushing {len(files)} files to {bucket.name}')
        for file in files:
            destination_blob_name = file.replace(media_folder, '', 1).lstrip('/')
            click.echo(f'Pushing {file} to {destination_blob_name}')
            blob = bucket.blob(destination_blob_name)
            # TODO: Read media files cache control value from settings
            blob.cache_control = 'private, max-age=180'
            blob.upload_from_filename(file)
            click.echo(f'{file} pushed successfully.')



from elementalcms.persistence.repositories import SnippetsRepository
from elementalcms.services import UseCaseResult, NoResult, Success
from elementalcms.core import MongoDbContext


class GetOne:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, _id) -> UseCaseResult:
        repo = SnippetsRepository(self.__db_context)
        snippet = repo.find_one(_id)
        if snippet is None:
            return NoResult()
        return Success(snippet)



import os
import pathlib
import setuptools

HERE = pathlib.Path(__file__).parent

LONG_DESCRIPTION = (HERE / "pypi.md").read_text()


REQUIREMENTS = [
    'Flask==2.0.2',
    'pycountry==20.7.3',
    'pymongo==3.12.1',
    'cloup==0.12.1',
    'deepdiff==5.6.0',
    'Flask-Babel==2.0.0',
    'google-cloud-storage==2.3.0',
]


def read(rel_path: str) -> str:
    here = os.path.abspath(os.path.dirname(__file__))
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    with open(os.path.join(here, rel_path)) as fp:
        return fp.read()


def get_version(rel_path: str) -> str:
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            # __version__ = "0.9"
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name="elemental-cms",
    version=get_version("elementalcms/__init__.py"),
    author="Paranoid Software",
    author_email="info@paranoid.software",
    license="MIT",
    description="Flask and MongoDB CMS for developers first.",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url='https://github.com/paranoid-software/elemental-cms',
    packages=setuptools.find_packages(exclude=['tests*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ],
    install_requires=REQUIREMENTS,
    include_package_data=True,
    python_requires='>=3.6',
    scripts=['elementalcms/bin/elemental-cms'],
)



import json

import pycountry
from bson import json_util
from flask import render_template, Blueprint, g, abort, current_app, request, redirect, session

from elementalcms.services import UseCaseResult
from elementalcms.services.pages import GetMe

admin = Blueprint('admin', __name__, template_folder='templates')


@admin.before_request
def before_request():
    lang_code = g.get('lang_code', None)
    if lang_code is None and current_app.config['LANGUAGE_MODE'] == 'single':
        return
    if lang_code is None:
        abort(404)
    if lang_code in current_app.config['LANGUAGES']:
        return
    return redirect(request.full_path.replace(lang_code,
                                              session.get('lang_code', current_app.config["DEFAULT_LANGUAGE"])))


@admin.url_defaults
def add_language_code(endpoint, values):
    values.setdefault('lang_code', session.get('langCode', current_app.config["DEFAULT_LANGUAGE"]))


@admin.url_value_preprocessor
def pull_lang_code(endpoint, values):

    lang_code = values.get('lang_code', None)
    if lang_code is None:
        return

    country = pycountry.languages.get(alpha_2=lang_code)
    if country is None:
        return

    g.lang_code = lang_code


@admin.route('/<slug>/edit/', methods=['GET'])
def index(slug: str, lang_code: str = None):
    admin_identity = session.get('adminIdentity', None)
    if not admin_identity:
        abort(401)

    if lang_code is None and current_app.config['LANGUAGE_MODE'] == 'single':
        lang_code = session.get('langCode', current_app.config['DEFAULT_LANGUAGE'])

    result: UseCaseResult = GetMe(current_app.config['CMS_DB_CONTEXT']).execute(slug,
                                                                                lang_code,
                                                                                True)
    if result.is_failure():
        abort(404)

    session['langCode'] = lang_code
    return render_template('admin/index.html',
                           page=json.loads(json_util.dumps(result.value())))



import os
import pathlib
from flask import Flask, Blueprint, request, send_from_directory, redirect, g, render_template_string, session, url_for
from flask_babel import Babel
from markupsafe import Markup

from elementalcms.core import ElementalContext
from elementalcms.extends import Applet, ActionsMapper

from elementalcms.persistence import MongoSessionInterface
from elementalcms.services.snippets import GetMe

from elementalcms.admin import admin
from elementalcms.presenter import presenter
from elementalcms.identity import identity

__version__ = "1.0.92"


class Elemental:

    def __init__(self, app: Flask, context: ElementalContext, applets: [Applet] = None):

        app.config.from_object(context.cms_core_context)

        # Passing DB Context to Blueprints
        app.config['CMS_DB_CONTEXT'] = context.cms_db_context

        translations_paths = [
            'translations',
            # Support for package translations
            f'{pathlib.Path(__file__).resolve().parent}/translations'
        ]
        app.config['BABEL_TRANSLATION_DIRECTORIES'] = ';'.join(translations_paths)

        admin.url_prefix = None if context.cms_core_context.LANGUAGE_MODE == 'single' else '/<lang_code>'
        app.register_blueprint(admin)
        presenter.url_prefix = None if context.cms_core_context.LANGUAGE_MODE == 'single' else '/<lang_code>'
        app.register_blueprint(presenter)

        app.register_blueprint(identity)

        app.register_blueprint(Blueprint('media', __name__, static_url_path='/media', static_folder=os.path.join(app.root_path, 'media')))

        for applet in applets or []:
            ActionsMapper(app).register_actions(applet.get_controllers())

        # Session support using MongoDB
        app.session_interface = MongoSessionInterface(context.cms_db_context)

        babel = Babel(app)

        @babel.localeselector
        def get_locale():
            if not g.get('lang_code', None):
                g.lang_code = request.accept_languages.best_match(context.cms_core_context.LANGUAGES)
            return g.lang_code

        @app.before_request
        def before_request():
            path = request.full_path

            if 'static' in path:
                if context.cms_core_context.DEBUG and ('static/admin' in path or 'static/presenter' in path):
                    path_parts = path.strip('?').split('/')
                    index = path_parts.index('static')
                    local_path = pathlib.Path(__file__).resolve().parent
                    return send_from_directory(local_path, '/'.join(path_parts[index:]), cache_timeout=360)
                return

            if request.full_path == '/?':
                if context.cms_core_context.LANGUAGE_MODE == 'single':
                    return
                return redirect(url_for('presenter.index',
                                        lang_code=session.get("langCode", context.cms_core_context.DEFAULT_LANGUAGE)))

            if request.full_path == '/?draft=1':
                if context.cms_core_context.LANGUAGE_MODE == 'single':
                    return
                return redirect(url_for('presenter.index',
                                        lang_code=session.get("langCode", context.cms_core_context.DEFAULT_LANGUAGE),
                                        draft=1))

        @app.context_processor
        def elemental_url_for_static_processor():
            def elemental_url_for_static(file_path=''):
                return f'{context.cms_core_context.STATIC_URL}/{file_path}'
            return dict(elemental_url_for_static=elemental_url_for_static)

        @app.context_processor
        def elemental_url_for_media_processor():
            def elemental_url_for_media(file_path=''):
                return f'{context.cms_core_context.MEDIA_URL}/{file_path}'
            return dict(elemental_url_for_media=elemental_url_for_media)

        @app.context_processor
        def elemental_url_for_slug_processor():
            def elemental_url_for_slug(lang_code=None, slug=None, **kwargs):
                url = url_for('presenter.index',
                              lang_code=None if context.cms_core_context.LANGUAGE_MODE == 'single' else
                              (lang_code or session.get('langCode', context.cms_core_context.DEFAULT_LANGUAGE)),
                              **kwargs)
                if slug is not None and slug.strip():
                    url = url_for('presenter.render',
                                  slug=slug,
                                  lang_code=None if context.cms_core_context.LANGUAGE_MODE == 'single' else
                                  (lang_code or session.get('langCode', context.cms_core_context.DEFAULT_LANGUAGE)),
                                  **kwargs)
                return url
            return dict(elemental_url_for_slug=elemental_url_for_slug)

        @app.context_processor
        def render_snippet_processor():
            def render_snippet(name):
                get_me_result = GetMe(context.cms_db_context).execute(name)
                if get_me_result.is_failure():
                    raise Exception(f'There are no snippets under the name {name}')
                content = render_template_string(f'<!--{name}-->\n{get_me_result.value()["content"]}')
                return Markup(content)
            return dict(render_snippet=render_snippet)

        @app.context_processor
        def lang_code_processor():
            code = ''
            if context.cms_core_context.LANGUAGE_MODE == 'multi':
                code = session.get('langCode', context.cms_core_context.DEFAULT_LANGUAGE)
            return dict(lang_code=code)



import click
from cloup import constraint, option, command, pass_context, argument
from cloup.constraints import RequireExactly

from .staticcommands import List, Collect, Delete


class Static(click.Group):

    def __init__(self):
        super(Static, self).__init__()
        self.name = 'static'
        self.add_command(self.list)
        self.add_command(self.collect)
        self.add_command(self.delete)

    @staticmethod
    @command(name='list',
             help='List static resources.')
    @option('--all',
            is_flag=True,
            help='List all static resources.')
    @option('--folder',
            '-f',
            nargs=1,
            multiple=True,
            help='Name of the folder to be listed. For example: list -f app/client-stack')
    @constraint(RequireExactly(1), ['all', 'folder'])
    @pass_context
    def list(ctx, **params):
        if params['all']:
            List(ctx).exec(None)
            return
        List(ctx).exec(params['folder'][0])

    @staticmethod
    @command(name='collect')
    @argument('pattern')
    @click.option("--ignore-internals",
                  is_flag=True,
                  show_default=True,
                  default=False,
                  help="Do not collect internal static resources.")
    @pass_context
    def collect(ctx, pattern, ignore_internals):
        """Collect static resources.

        You must use a search PATTERN to identify the files to be collected.

        Samples (assuming static is our local static folder):

        \b
        static/*.*
        static/*.txt
        static/*.svg
        static/**/images
        """
        Collect(ctx).exec(pattern, ignore_internals)

    @staticmethod
    @command(name='delete')
    @argument('file')
    @pass_context
    def delete(ctx, file):
        """Delete especified file from GCS."""
        Delete(ctx).exec(file)



from flask.sessions import SessionMixin
from werkzeug.datastructures import CallbackDict


class MongoSession(CallbackDict, SessionMixin):

    def __init__(self, initial=None, sid=None):
        CallbackDict.__init__(self, initial)
        self.sid = sid
        self.modified = False
